
import sys, pygame, os, random, time

# Constants
size = width, height = 1080, 720
black = (0, 0, 0)
white = (255, 255, 255)
blue = (0, 0, 255)
fps = 30
ship_width = ship_height = 40
box_width = box_height = 30
door_width = 200
door_height = 10

# Classes
class Ship(pygame.sprite.Sprite):
    def __init__(self, x, y, player):
        pygame.sprite.Sprite.__init__(self)
        self.player = player
        self.speed = 4
        self.image = pygame.transform.scale(pygame.image.load("image/Ness.jpg").convert(), (ship_width, ship_height))
        self.rect = self.image.get_rect()
        self.rect = self.rect.move(x, y)
        self.current_injuries = 0
        self.people_count = 0
        self.boxes_left = 10
        self.new_box = False

    def update(self, remaining_boxes, people_group, box_group):
        key = pygame.key.get_pressed()
        # Ship movement
        if self.player == "left":
            if key[pygame.K_UP]:
                self.rect.y -= self.speed
            if key[pygame.K_DOWN]:
                self.rect.y += self.speed
            if key[pygame.K_LEFT]:
                self.rect.x -= self.speed
            if key[pygame.K_RIGHT]:
                self.rect.x += self.speed
            # Right Boundary
            if self.rect.x > width-self.rect.width:
                self.rect.x = width-self.rect.width
            # Left Boundary
            if self.rect.x < 0:
                self.rect.x = 0

            collisions = pygame.sprite.spritecollide(self, people_group, True)
            for key in collisions:
                self.current_injuries += key.injuries_added
                self.people_count -= 1

            collisions = pygame.sprite.spritecollide(self, box_group, True)
            for key in collisions:
                self.boxes_left -= 1
                self.new_box = True

        # Screen Boundaries
        if self.rect.y < height/15:
            self.rect.y = height/15
        if self.rect.y > height-self.rect.height:
            self.rect.y = height-self.rect.height

class Door(pygame.sprite.Sprite):
    def __init__(self, xval, yval):
        pygame.sprite.Sprite.__init__(self)
        self.image = pygame.Surface((door_width, door_height)).convert()
        self.rect = pygame.Rect(xval, yval, door_width, door_height)
        self.image.fill(blue)

class People(pygame.sprite.Sprite):
    def __init__(self, x, y, speed):
        pygame.sprite.Sprite.__init__(self)
        self.image = pygame.transform.scale(pygame.image.load("image/Swaglax.jpg").convert(), (ship_width, ship_height))
        self.rect = self.image.get_rect()
        self.rect = self.rect.move(x, y)
        self.speed = speed
        self.injuries_added = 1

    def update(self, height, people_group):
        self.rect = self.rect.move(self.speed)
        #ybounds
        if (self.rect.top - 5) < height/15 or (self.rect.bottom + 5) > height:
            self.speed[1] = -self.speed[1]
        #xbounds
        if (self.rect.left - 5) < 0 or (self.rect.right + 5) > width:
            self.speed[0] = -self.speed[0]

class Box(pygame.sprite.Sprite):
    def __init__(self, xval, yval):
        pygame.sprite.Sprite.__init__(self)
        self.speed = 5
        self.image = pygame.Surface((box_width, box_height)).convert()
        self.rect = pygame.Rect(xval, yval, box_width, box_height)
        self.image.fill(blue)

class Text1(pygame.sprite.Sprite):
    def __init__(self, size, color, position, font):
        pygame.sprite.Sprite.__init__(self)
        self.color = color
        self.position = position
        self.font = pygame.font.Font(font, size)

    def _update(self, current_injuries):
        text = "Injuries: " + str(current_injuries)
        self.image = self.font.render(str(text), 1, self.color)
        self.rect = self.image.get_rect()
        self.rect.move_ip(self.position[0]-self.rect.width/2, self.position[1])

class Score(Text1):
    def __init__(self, size, color, position, f=None):
        super(Score, self).__init__(size, color, position, font=f)

    def update(self, density):
        self._update(density)

class Text2(pygame.sprite.Sprite):
    def __init__(self, size, color, position, font):
        pygame.sprite.Sprite.__init__(self)
        self.color = color
        self.position = position
        self.font = pygame.font.Font(font, size)

    def _update(self, people_count):
        text = "Total People: " + str(people_count)
        self.image = self.font.render(str(text), 1, self.color)
        self.rect = self.image.get_rect()
        self.rect.move_ip(self.position[0]-self.rect.width/2, self.position[1])

class PeopleCount(Text2):
    def __init__(self, size, color, position, f=None):
        super(PeopleCount, self).__init__(size, color, position, font=f)

    def update(self, density):
        self._update(density)

class Text3(pygame.sprite.Sprite):
    def __init__(self, size, color, position, font):
        pygame.sprite.Sprite.__init__(self)
        self.color = color
        self.position = position
        self.font = pygame.font.Font(font, size)

    def _update(self, box_count):
        text = "Boxes Left: " + str(box_count)
        self.image = self.font.render(str(text), 1, self.color)
        self.rect = self.image.get_rect()
        self.rect.move_ip(self.position[0]-self.rect.width/2, self.position[1])

class BoxCount(Text3):
    def __init__(self, size, color, position, f=None):
        super(BoxCount, self).__init__(size, color, position, font=f)

    def update(self, density):
        self._update(density)

def gen_random():
    xval_density = []
    for _ in range(10):
        xval_density.append((random.randrange(0, (width) - box_width), random.randrange(height/15, height - box_height)))
    return xval_density


def main():
    # Initialize variables
    pygame.display.set_caption('Mason de LOL: The Escape Demo')
    screen = pygame.display.set_mode(size, pygame.SRCALPHA)
    first_time = True
    clock = pygame.time.Clock()

    beg_time = pygame.time.get_ticks()
    timer = 0
    play = outro = False
    title_screen = game_start = True
    xval_density = gen_random()

    # Create game objects
    player = Ship((width/2) - (ship_width/2), height/2 - (ship_height), 'left')
    left_door = Door(width/4 - door_width/2, height/15)
    right_door = Door(3*width/4 - door_width/2, height/15)
    box = Box(xval_density[0][0], xval_density[0][1])
    horz_partition = pygame.Surface((width, 5))
    score = Score(40, black, (width/4, height/50))
    total_people = PeopleCount (40, black, (3*width/4, height/50))
    boxes_left = BoxCount (40, black, (width/2, height/50))

    # Create groups
    ship_group = pygame.sprite.Group()
    ship_group.add(player)

    door_group = pygame.sprite.Group()
    door_group.add(left_door, right_door)

    left_people_group = pygame.sprite.Group()
    right_people_group = pygame.sprite.Group()

    injuries_group = pygame.sprite.Group()
    injuries_group.add(score)

    total_people_group = pygame.sprite.Group()
    total_people_group.add(total_people)

    box_group = pygame.sprite.Group()
    box_group.add(box)

    boxes_left_group = pygame.sprite.Group()
    boxes_left_group.add(boxes_left)


    while game_start:
        # Intro loop
        while title_screen:
            # Print background
            titlebackground = pygame.Surface((width, height)).convert()
            titlebackground.fill(white)
            titlebackrect = titlebackground.get_rect()
            titlebackrect = titlebackrect.move(0,0)

            # Creating our text objects
            titlefont = pygame.font.Font(None, 75)
            title = titlefont.render("Mason de LOL: The Escape Demo", 1, black)
            titlepos = title.get_rect()
            titlepos.centerx = screen.get_rect().centerx
            titlepos.centery = screen.get_rect().centery/2.5

            screen.blit(titlebackground, titlebackrect)
            screen.blit(title, titlepos)

            startfont = pygame.font.Font(None, 75)
            start = startfont.render("-Click to begin-", 1, black)
            startpos = start.get_rect()
            startpos.centerx = screen.get_rect().centerx
            startpos.centery = screen.get_rect().centery/.9

            # Blinking Test: Click here to start
            cur_time = pygame.time.get_ticks()
            if((cur_time - beg_time)%1000) < 500:
                screen.blit(start, startpos)

            # Checks if window exit button pressed
            for event in pygame.event.get():
                if event.type == pygame.QUIT: sys.exit()
                elif event.type == pygame.MOUSEBUTTONDOWN or pygame.key.get_pressed()[pygame.K_RETURN] != 0:
                    screen.blit(start, startpos)
                    pygame.display.flip()
                    pygame.time.wait(1500)
                    title_screen = outro = False
                    play = True
            # Limits frames per iteration of while loop
            clock.tick(fps)
            # Writes to main surface
            pygame.display.flip()

        # Game loop
        while play:
            # Checks if window exit button pressed
            for event in pygame.event.get():
                if event.type == pygame.QUIT: sys.exit()

                 # Keypresses
                elif event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_ESCAPE:
                        pygame.quit()
                        sys.exit()

            # Print background
            if first_time == True:
                titlebackground = pygame.Surface((width, height)).convert()
                titlebackground.fill(white)
                titlebackrect = titlebackground.get_rect()
                titlebackrect = titlebackrect.move(0,0)
            screen.blit(titlebackground, titlebackrect)

            # Update Groups
            ship_group.update(boxes_left, left_people_group, box_group)
            ship_group.update(boxes_left, right_people_group, box_group)
            door_group.update()
            left_people_group.update(height, right_people_group)
            right_people_group.update(height, left_people_group)
            injuries_group.update(player.current_injuries)
            total_people_group.update(player.people_count)
            box.update(xval_density[0][0], xval_density[0][1], ship_group)
            boxes_left_group.update(player.boxes_left)

            # Adding Pills
            if timer % 60 == 0 and first_time == False:
                left_person = People(width/4 - door_width/2 + random.randrange(0, door_width), height/15, [random.randrange(-1,1) * 5 + random.randrange(-10,10), 5])
                right_person = People(3*width/4 - door_width/2 + random.randrange(0, door_width), height/15, [random.randrange(-1,1) * 5 + random.randrange(-10,10), 5])
                left_people_group.add(left_person)
                right_people_group.add(right_person)
                player.people_count +=2

            # Adding Boxes
            if player.new_box == True and player.boxes_left > 0:
                xval_density = gen_random()
                box = Box(xval_density[0][0], xval_density[0][1])
                box_group.add(box)
                player.new_box = False


            first_time = False
            timer += 1

            # Printing Groups
            ship_group.draw(screen)
            door_group.draw(screen)
            left_people_group.draw(screen)
            right_people_group.draw(screen)
            screen.blit(horz_partition, (0, height/15))
            injuries_group.draw(screen)
            total_people_group.draw(screen)
            box_group.draw(screen)
            boxes_left_group.draw(screen)

            # Limits frames per iteration of while loop
            clock.tick(fps)
            # Writes to main surface
            pygame.display.flip()

            if player.boxes_left == 0 or player.current_injuries >=5:
                outro = True

            while outro:
                # Print background
                titlebackground = pygame.Surface((width, height)).convert()
                titlebackground.fill(white)
                titlebackrect = titlebackground.get_rect()
                titlebackrect = titlebackrect.move(0,0)

                for p in left_people_group:
                    p.kill()

                for p in right_people_group:
                    p.kill()

                # Creating our text objects
                if player.boxes_left == 0:
                    titlefont = pygame.font.Font(None, 200)
                    title = titlefont.render("You Win!", 1, black)
                    titlepos = title.get_rect()
                    titlepos.centerx = screen.get_rect().centerx
                    titlepos.centery = screen.get_rect().centery/2.5

                if player.current_injuries >= 5:
                    titlefont = pygame.font.Font(None, 200)
                    title = titlefont.render("You Lose!", 1, black)
                    titlepos = title.get_rect()
                    titlepos.centerx = screen.get_rect().centerx
                    titlepos.centery = screen.get_rect().centery/2.5

                screen.blit(titlebackground, titlebackrect)
                screen.blit(title, titlepos)

                startfont = pygame.font.Font(None, 75)
                start = startfont.render("-Click here to restart-", 1, black)
                startpos = start.get_rect()
                startpos.centerx = screen.get_rect().centerx
                startpos.centery = screen.get_rect().centery/.9

                #Blinking Test: Click here to start
                cur_time = pygame.time.get_ticks()
                if((cur_time - beg_time)%1000) < 500:
                    screen.blit(start, startpos)

                # Checks if window exit button pressed
                for event in pygame.event.get():
                    if event.type == pygame.QUIT: sys.exit()
                    elif event.type == pygame.MOUSEBUTTONDOWN or pygame.key.get_pressed()[pygame.K_RETURN] != 0:
                        screen.blit(start, startpos)
                        pygame.display.flip()
                        pygame.time.wait(1500)
                        player.boxes_left = 10
                        title_screen = True
                        player.current_injuries = 0
                        player.people_count = 0
                        outro = play = False
                # Limits frames per iteration of while loop
                clock.tick(fps)
                # Writes to main surface
                pygame.display.flip()


if __name__ == "__main__":
    # Force static position of screen
    os.environ['SDL_VIDEO_CENTERED'] = '1'

    # Runs imported module
    pygame.init()

    main()